
import os
import sys
import time

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from sface.config import config
from sface.logwindow import LogWindow
from sface.screens.configscreen import ConfigScreen
from sface.screens.mainscreen import MainScreen
from sface.screens.rspecscreen import RSpecScreen

class Nav(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        
        self.title = QLabel("", self)
        self.link1 = QLabel("", self)
        self.link1.setAlignment(Qt.AlignRight)

        self.link2 = QLabel("", self)
        self.link2.setAlignment(Qt.AlignRight)
        
        hlayout = QHBoxLayout()
        hlayout.addWidget(self.title)
        hlayout.addStretch()
        hlayout.addWidget(QLabel("Go to: ", self))
        hlayout.addWidget(self.link1)
        hlayout.addWidget(self.link2)
        self.setLayout(hlayout)

    def setTitle(self, title):
        self.title.setText(title)

    def setLinks(self, link1, link2):
        self.link1.setText(link1)
        self.link2.setText(link2)


class Status(QLabel):
    def __init__(self, parent=None):
        QLabel.__init__(self, "", parent)
        self.sliceUpdateDate()

    def set(self, msg, timeout):
        self.setText(msg)
        if timeout:
            QTimer.singleShot(timeout, self.reset)

    def sliceUpdateDate(self):
        rspec_file = config.getSliceRSpecFile()
        if not os.path.exists(rspec_file):
            return

        creation_time = os.stat(rspec_file).st_ctime
        last_update = time.ctime(creation_time)
        self.set("Slice data last updated on %s" % last_update, timeout=None)

    def reset(self):
        self.setText("")
        QTimer.singleShot(1500, self.sliceUpdateDate)

class MainWindow(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)

        self.logWindow = LogWindow(self)

        self.pix = QLabel(self)
        self.config_screen = ConfigScreen(self)
        self.main_screen = MainScreen(self)
        self.rspec_screen = RSpecScreen(self)

        self.screens = QStackedWidget(self)
        self.screens.addWidget(self.main_screen)
        self.screens.addWidget(self.config_screen)
        self.screens.addWidget(self.rspec_screen)
        self.screens.addWidget(self.pix)
        self.next_screen = None

        self.nav = Nav(self)
        self.nav.setTitle(self.main_screen.getTitleText())
        self.nav.setLinks(self.rspec_screen.getLinkText(), self.config_screen.getLinkText())

        self.status = Status(self)
        self.log = QLabel("<a href='showlog'>Show Log</a>", self)

        hlayout = QHBoxLayout()
        hlayout.addWidget(self.status)
        hlayout.addStretch()
        hlayout.addWidget(self.log)

        layout = QVBoxLayout()
        layout.addWidget(self.nav)
        layout.addWidget(self.screens)
        layout.addLayout(hlayout)
        self.setLayout(layout)
        self.resize(800, 600)

        self.connect(self.nav.link1, SIGNAL('linkActivated(QString)'),
                     self.animateToScreen)
        self.connect(self.nav.link2, SIGNAL('linkActivated(QString)'),
                     self.animateToScreen)
        self.connect(self.log, SIGNAL('linkActivated(QString)'),
                     self.showLogWindow)

    def redirectOutputToLog(self):
        self.logWindow.redirectOutput()

    def showLogWindow(self, link):
        self.logWindow.show()
        self.logWindow.resize(500, 500)
        self.logWindow.raise_()
        self.logWindow.activateWindow()

    def animatePixmap(self, y):
        self.pix.move(0, y)

    def animateToScreen(self, link):
        if link == self.config_screen.name:
            self.next_screen = self.config_screen
        elif link == self.main_screen.name:
            self.next_screen = self.main_screen
        elif link == self.rspec_screen.name:
            self.next_screen = self.rspec_screen
        
        pixmap = QPixmap(self.screens.size())
        self.screens.currentWidget().render(pixmap)
        self.screens.setCurrentWidget(self.pix)
        self.pix.setPixmap(pixmap)

        timeLine = QTimeLine(500, self)
        timeLine.setFrameRange(0, self.screens.height());
        self.connect(timeLine, SIGNAL('frameChanged(int)'), self.animatePixmap)
        self.connect(timeLine, SIGNAL('finished()'), self.toNextScreen)
        timeLine.start()


    def toNextScreen(self):
        if self.next_screen == self.config_screen:
            self.toConfigScreen()
        elif self.next_screen == self.main_screen:
            self.toMainScreen()
        elif self.next_screen == self.rspec_screen:
            self.toRSpecScreen()

    def toConfigScreen(self):
        self.screens.setCurrentWidget(self.config_screen)
        self.nav.setLinks(self.main_screen.getLinkText(), self.rspec_screen.getLinkText())
        self.nav.setTitle(self.config_screen.getTitleText())

    def toMainScreen(self):
        self.screens.setCurrentWidget(self.main_screen)
        self.nav.setLinks(self.rspec_screen.getLinkText(), self.config_screen.getLinkText())
        self.nav.setTitle(self.main_screen.getTitleText())

    def toRSpecScreen(self):
        self.screens.setCurrentWidget(self.rspec_screen)
        self.nav.setLinks(self.main_screen.getLinkText(), self.config_screen.getLinkText())
        self.nav.setTitle(self.rspec_screen.getTitleText())

    def setStatus(self, msg, timeout):
        self.status.set(msg, timeout)

